@@ -1,5 +1,5 @@
 /*  Lzip - Data compressor based on the LZMA algorithm
-    Copyright (C) 2008, 2009, 2010 Antonio Diaz Diaz.
+    Copyright (C) 2008, 2009, 2010, 2011, 2012 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -75,10 +75,10 @@
 
 namespace {
 
+const char * const Program_name = "Lzip";
+const char * const program_name = "lzip";
+const char * const program_year = "2012";
 const char * invocation_name = 0;
-const char * const Program_name    = "Lzip";
-const char * const program_name    = "lzip";
-const char * const program_year    = "2010";
 
 #ifdef O_BINARY
 const int o_binary = O_BINARY;
@@ -102,71 +102,78 @@ enum Mode { m_compress, m_decompress, m_test };
 std::string output_filename;
 int outfd = -1;
 int verbosity = 0;
-mode_t outfd_mode = S_IRUSR | S_IWUSR;
+const mode_t usr_rw = S_IRUSR | S_IWUSR;
+const mode_t all_rw = usr_rw | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
+mode_t outfd_mode = usr_rw;
 bool delete_output_on_interrupt = false;
 
 
-void show_help() throw()
+void show_help()
   {
   std::printf( "%s - Data compressor based on the LZMA algorithm.\n", Program_name );
   std::printf( "\nUsage: %s [options] [files]\n", invocation_name );
-  std::printf( "\nOptions:\n" );
-  std::printf( "  -h, --help                 display this help and exit\n" );
-  std::printf( "  -V, --version              output version information and exit\n" );
-  std::printf( "  -b, --member-size=<n>      set member size limit in bytes\n" );
-  std::printf( "  -c, --stdout               send output to standard output\n" );
-  std::printf( "  -d, --decompress           decompress\n" );
-  std::printf( "  -f, --force                overwrite existing output files\n" );
-  std::printf( "  -k, --keep                 keep (don't delete) input files\n" );
-  std::printf( "  -m, --match-length=<n>     set match length limit in bytes [36]\n" );
-  std::printf( "  -o, --output=<file>        if reading stdin, place the output into <file>\n" );
-  std::printf( "  -q, --quiet                suppress all messages\n" );
-  std::printf( "  -s, --dictionary-size=<n>  set dictionary size limit in bytes [8MiB]\n" );
-  std::printf( "  -S, --volume-size=<n>      set volume size limit in bytes\n" );
-  std::printf( "  -t, --test                 test compressed file integrity\n" );
-  std::printf( "  -v, --verbose              be verbose (a 2nd -v gives more)\n" );
-  std::printf( "  -0 .. -9                   set compression level [default 6]\n" );
-  std::printf( "      --fast                 alias for -0\n" );
-  std::printf( "      --best                 alias for -9\n" );
-  std::printf( "If no file names are given, %s compresses or decompresses\n", program_name );
-  std::printf( "from standard input to standard output.\n" );
-  std::printf( "Numbers may be followed by a multiplier: k = kB = 10^3 = 1000,\n" );
-  std::printf( "Ki = KiB = 2^10 = 1024, M = 10^6, Mi = 2^20, G = 10^9, Gi = 2^30, etc...\n" );
-  std::printf( "\nReport bugs to lzip-bug@nongnu.org\n" );
-  std::printf( "Lzip home page: http://www.nongnu.org/lzip/lzip.html\n" );
+  std::printf( "\nOptions:\n"
+               "  -h, --help                     display this help and exit\n"
+               "  -V, --version                  output version information and exit\n"
+               "  -b, --member-size=<bytes>      set member size limit in bytes\n"
+               "  -c, --stdout                   send output to standard output\n"
+               "  -d, --decompress               decompress\n"
+               "  -f, --force                    overwrite existing output files\n"
+               "  -F, --recompress               force recompression of compressed files\n"
+               "  -k, --keep                     keep (don't delete) input files\n"
+               "  -m, --match-length=<bytes>     set match length limit in bytes [36]\n"
+               "  -o, --output=<file>            if reading stdin, place the output into <file>\n"
+               "  -q, --quiet                    suppress all messages\n"
+               "  -s, --dictionary-size=<bytes>  set dictionary size limit in bytes [8MiB]\n"
+               "  -S, --volume-size=<bytes>      set volume size limit in bytes\n"
+               "  -t, --test                     test compressed file integrity\n"
+               "  -v, --verbose                  be verbose (a 2nd -v gives more)\n"
+               "  -0 .. -9                       set compression level [default 6]\n"
+               "      --fast                     alias for -0\n"
+               "      --best                     alias for -9\n"
+               "If no file names are given, lzip compresses or decompresses\n"
+               "from standard input to standard output.\n"
+               "Numbers may be followed by a multiplier: k = kB = 10^3 = 1000,\n"
+               "Ki = KiB = 2^10 = 1024, M = 10^6, Mi = 2^20, G = 10^9, Gi = 2^30, etc...\n"
+               "The bidimensional parameter space of LZMA can't be mapped to a linear\n"
+               "scale optimal for all files. If your files are large, very repetitive,\n"
+               "etc, you may need to use the --match-length and --dictionary-size\n"
+               "options directly to achieve optimal performance.\n"
+               "\nReport bugs to lzip-bug@nongnu.org\n"
+               "Lzip home page: http://www.nongnu.org/lzip/lzip.html\n" );
   }
 
 
-void show_version() throw()
+void show_version()
   {
   std::printf( "%s %s\n", Program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
-  std::printf( "This is free software: you are free to change and redistribute it.\n" );
-  std::printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
+  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
+               "This is free software: you are free to change and redistribute it.\n"
+               "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
 
-const char * format_num( long long num ) throw()
+const char * format_num( long long num )
   {
   const char * const prefix[8] =
     { "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi" };
-  enum { buf_size = 16 };
+  enum { buf_size = 16, factor = 1024 };
   static char buf[buf_size];
   const char *p = "";
-  const int factor = 1024;
+  bool exact = ( num % factor == 0 );
 
   for( int i = 0; i < 8 && ( llabs( num ) > 9999 ||
-       ( llabs( num ) >= factor && num % factor == 0 ) ); ++i )
-    { num /= factor; p = prefix[i]; }
+                             ( exact && llabs( num ) >= factor ) ); ++i )
+    { num /= factor; if( num % factor != 0 ) exact = false; p = prefix[i]; }
   snprintf( buf, buf_size, "%lld %s", num, p );
   return buf;
   }
 
 
-long long getnum( const char * const ptr, const int bs = 0,
+long long getnum( const char * const ptr,
                   const long long llimit = LLONG_MIN + 1,
-                  const long long ulimit = LLONG_MAX ) throw()
+                  const long long ulimit = LLONG_MAX )
   {
   errno = 0;
   char *tail;
@@ -185,9 +192,6 @@ long long getnum( const char * const ptr, const int bs = 0,
     switch( tail[0] )
       {
       case ' ': break;
-      case 'b': if( bs > 0 ) { factor = bs; exponent = 1; }
-                else bad_multiplier = true;
-                break;
       case 'Y': exponent = 8; break;
       case 'Z': exponent = 7; break;
       case 'E': exponent = 6; break;
@@ -222,18 +226,18 @@ long long getnum( const char * const ptr, const int bs = 0,
   }
 
 
-int get_dict_size( const char * const arg ) throw()
+int get_dict_size( const char * const arg )
   {
   char *tail;
   int bits = std::strtol( arg, &tail, 0 );
   if( bits >= min_dictionary_bits &&
       bits <= max_dictionary_bits && *tail == 0 )
     return ( 1 << bits );
-  return getnum( arg, 0, min_dictionary_size, max_dictionary_size );
+  return getnum( arg, min_dictionary_size, max_dictionary_size );
   }
 
 
-int extension_index( const std::string & name ) throw()
+int extension_index( const std::string & name )
   {
   for( int i = 0; known_extensions[i].from; ++i )
     {
@@ -248,13 +252,13 @@ int extension_index( const std::string & name ) throw()
 
 int open_instream( const std::string & name, struct stat * const in_statsp,
                    const Mode program_mode, const int eindex,
-                   const bool force, const bool to_stdout ) throw()
+                   const bool recompress, const bool to_stdout )
   {
   int infd = -1;
-  if( program_mode == m_compress && !force && eindex >= 0 )
+  if( program_mode == m_compress && !recompress && eindex >= 0 )
     {
     if( verbosity >= 0 )
-      std::fprintf( stderr, "%s: Input file `%s' already has `%s' suffix.\n",
+      std::fprintf( stderr, "%s: Input file '%s' already has '%s' suffix.\n",
                     program_name, name.c_str(),
                     known_extensions[eindex].from );
     }
@@ -264,21 +268,23 @@ int open_instream( const std::string & name, struct stat * const in_statsp,
     if( infd < 0 )
       {
       if( verbosity >= 0 )
-        std::fprintf( stderr, "%s: Can't open input file `%s': %s.\n",
+        std::fprintf( stderr, "%s: Can't open input file '%s': %s.\n",
                       program_name, name.c_str(), std::strerror( errno ) );
       }
     else
       {
       const int i = fstat( infd, in_statsp );
       const mode_t & mode = in_statsp->st_mode;
-      if( i < 0 || !( S_ISREG( mode ) || ( to_stdout &&
-                      ( S_ISFIFO( mode ) || S_ISSOCK( mode ) ||
-                        S_ISBLK( mode ) || S_ISCHR( mode ) ) ) ) )
+      const bool can_read = ( i == 0 &&
+                              ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
+                                S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
+      if( i != 0 || ( !S_ISREG( mode ) && ( !to_stdout || !can_read ) ) )
         {
         if( verbosity >= 0 )
-          std::fprintf( stderr, "%s: Input file `%s' is not a regular file%s.\n",
+          std::fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
                         program_name, name.c_str(),
-                        to_stdout ? "" : " and `--stdout' was not specified" );
+                        ( can_read && !to_stdout ) ?
+                        " and '--stdout' was not specified" : "" );
         close( infd );
         infd = -1;
         }
@@ -288,7 +294,7 @@ int open_instream( const std::string & name, struct stat * const in_statsp,
   }
 
 
-void set_c_outname( const std::string & name, const bool multifile ) throw()
+void set_c_outname( const std::string & name, const bool multifile )
   {
   output_filename = name;
   if( multifile ) output_filename += "00001";
@@ -296,7 +302,7 @@ void set_c_outname( const std::string & name, const bool multifile ) throw()
   }
 
 
-void set_d_outname( const std::string & name, const int i ) throw()
+void set_d_outname( const std::string & name, const int i )
   {
   if( i >= 0 )
     {
@@ -309,13 +315,13 @@ void set_d_outname( const std::string & name, const int i ) throw()
       }
     }
   output_filename = name; output_filename += ".out";
-  if( verbosity >= 0 )
-    std::fprintf( stderr, "%s: Can't guess original name for `%s' -- using `%s'.\n",
+  if( verbosity >= 1 )
+    std::fprintf( stderr, "%s: Can't guess original name for '%s' -- using '%s'.\n",
                   program_name, name.c_str(), output_filename.c_str() );
   }
 
 
-bool open_outstream( const bool force ) throw()
+bool open_outstream( const bool force )
   {
   int flags = O_CREAT | O_WRONLY | o_binary;
   if( force ) flags |= O_TRUNC; else flags |= O_EXCL;
@@ -324,17 +330,17 @@ bool open_outstream( const bool force ) throw()
   if( outfd < 0 && verbosity >= 0 )
     {
     if( errno == EEXIST )
-      std::fprintf( stderr, "%s: Output file `%s' already exists, skipping.\n",
+      std::fprintf( stderr, "%s: Output file '%s' already exists, skipping.\n",
                     program_name, output_filename.c_str() );
     else
-      std::fprintf( stderr, "%s: Can't create output file `%s': %s.\n",
+      std::fprintf( stderr, "%s: Can't create output file '%s': %s.\n",
                     program_name, output_filename.c_str(), std::strerror( errno ) );
     }
   return ( outfd >= 0 );
   }
 
 
-bool check_tty( const int infd, const Mode program_mode ) throw()
+bool check_tty( const int infd, const Mode program_mode )
   {
   if( program_mode == m_compress && outfd >= 0 && isatty( outfd ) )
     {
@@ -351,16 +357,16 @@ bool check_tty( const int infd, const Mode program_mode ) throw()
   }
 
 
-void cleanup_and_fail( const int retval ) throw()
+void cleanup_and_fail( const int retval )
   {
   if( delete_output_on_interrupt )
     {
     delete_output_on_interrupt = false;
     if( verbosity >= 0 )
-      std::fprintf( stderr, "%s: Deleting output file `%s', if it exists.\n",
-               program_name, output_filename.c_str() );
+      std::fprintf( stderr, "%s: Deleting output file '%s', if it exists.\n",
+                    program_name, output_filename.c_str() );
     if( outfd >= 0 ) { close( outfd ); outfd = -1; }
-    if( std::remove( output_filename.c_str() ) != 0 )
+    if( std::remove( output_filename.c_str() ) != 0 && errno != ENOENT )
       show_error( "WARNING: deletion of output file (apparently) failed." );
     }
   std::exit( retval );
@@ -370,30 +376,26 @@ void cleanup_and_fail( const int retval ) throw()
      // Set permissions, owner and times.
 void close_and_set_permissions( const struct stat * const in_statsp )
   {
-  bool error = false;
+  bool warning = false;
   if( in_statsp )
     {
-    if( fchmod( outfd, in_statsp->st_mode ) != 0 ||
-        ( fchown( outfd, in_statsp->st_uid, in_statsp->st_gid ) != 0 &&
-          errno != EPERM ) ) error = true;
     // fchown will in many cases return with EPERM, which can be safely ignored.
+    if( ( fchown( outfd, in_statsp->st_uid, in_statsp->st_gid ) != 0 &&
+          errno != EPERM ) ||
+        fchmod( outfd, in_statsp->st_mode ) != 0 ) warning = true;
     }
-  if( close( outfd ) == 0 ) outfd = -1;
-  else cleanup_and_fail( 1 );
+  if( close( outfd ) != 0 ) cleanup_and_fail( 1 );
+  outfd = -1;
   delete_output_on_interrupt = false;
-  if( !in_statsp ) return;
-  if( !error )
+  if( in_statsp )
     {
     struct utimbuf t;
     t.actime = in_statsp->st_atime;
     t.modtime = in_statsp->st_mtime;
-    if( utime( output_filename.c_str(), &t ) != 0 ) error = true;
+    if( utime( output_filename.c_str(), &t ) != 0 ) warning = true;
     }
-  if( error )
-    {
+  if( warning && verbosity >= 1 )
     show_error( "Can't change output file attributes." );
-    cleanup_and_fail( 1 );
-    }
   }
 
 
@@ -414,14 +416,14 @@ int compress( const long long member_size, const long long volume_size,
               const Lzma_options & encoder_options, const int infd,
               const Pretty_print & pp, const struct stat * const in_statsp )
   {
-  if( verbosity >= 1 ) pp();
+  int retval = 0;
   File_header header;
   header.set_magic();
+  if( verbosity >= 1 ) pp();
   if( !header.dictionary_size( encoder_options.dictionary_size ) ||
       encoder_options.match_len_limit < min_match_len_limit ||
       encoder_options.match_len_limit > max_match_len )
     internal_error( "invalid argument to encoder" );
-  int retval = 0;
 
   try {
     Matchfinder matchfinder( header.dictionary_size(),
@@ -458,7 +460,7 @@ int compress( const long long member_size, const long long volume_size,
     if( retval == 0 && verbosity >= 1 )
       {
       if( in_size <= 0 || out_size <= 0 )
-        std::fprintf( stderr, "No data compressed.\n" );
+        std::fprintf( stderr, " no data compressed.\n" );
       else
         std::fprintf( stderr, "%6.3f:1, %6.3f bits/byte, "
                               "%5.2f%% saved, %lld in, %lld out.\n",
@@ -482,10 +484,10 @@ int fcompress( const long long member_size, const long long volume_size,
                const int infd,
                const Pretty_print & pp, const struct stat * const in_statsp )
   {
-  if( verbosity >= 1 ) pp();
+  int retval = 0;
   File_header header;
   header.set_magic();
-  int retval = 0;
+  if( verbosity >= 1 ) pp();
 
   try {
     Fmatchfinder fmatchfinder( infd );
@@ -521,7 +523,7 @@ int fcompress( const long long member_size, const long long volume_size,
     if( retval == 0 && verbosity >= 1 )
       {
       if( in_size <= 0 || out_size <= 0 )
-        std::fprintf( stderr, "No data compressed.\n" );
+        std::fprintf( stderr, " no data compressed.\n" );
       else
         std::fprintf( stderr, "%6.3f:1, %6.3f bits/byte, "
                               "%5.2f%% saved, %lld in, %lld out.\n",
@@ -541,6 +543,42 @@ int fcompress( const long long member_size, const long long volume_size,
   }
 
 
+unsigned char xdigit( const int value )
+  {
+  if( value >= 0 && value <= 9 ) return '0' + value;
+  if( value >= 10 && value <= 15 ) return 'A' + value - 10;
+  return 0;
+  }
+
+
+void show_trailing_garbage( const uint8_t * const data, const int size,
+                            const Pretty_print & pp, const bool all )
+  {
+  std::string garbage_msg;
+  if( !all ) garbage_msg = "first bytes of ";
+  garbage_msg += "trailing garbage found = ";
+  bool text = true;
+  for( int i = 0; i < size; ++i )
+    if( !std::isprint( data[i] ) ) { text = false; break; }
+  if( text )
+    {
+    garbage_msg += '\'';
+    garbage_msg.append( (const char *)data, size );
+    garbage_msg += '\'';
+    }
+  else
+    {
+    for( int i = 0; i < size; ++i )
+      {
+      if( i > 0 ) garbage_msg += ' ';
+      garbage_msg += xdigit( data[i] >> 4 );
+      garbage_msg += xdigit( data[i] & 0x0F );
+      }
+    }
+  pp( garbage_msg.c_str() );
+  }
+
+
 int decompress( const int infd, const Pretty_print & pp, const bool testing )
   {
   int retval = 0;
@@ -552,18 +590,22 @@ int decompress( const int infd, const Pretty_print & pp, const bool testing )
       {
       File_header header;
       rdec.reset_member_position();
-      for( int i = 0; i < File_header::size; ++i )
-        header.data[i] = rdec.get_byte();
+      const int size = rdec.read_data( header.data, File_header::size );
       if( rdec.finished() )			// End Of File
         {
-        if( first_member ) { pp( "Error reading member header" ); retval = 1; }
+        if( first_member )
+          { pp( "Error reading member header" ); retval = 1; }
+        else if( verbosity >= 4 && size > 0 )
+          show_trailing_garbage( header.data, size, pp, true );
         break;
         }
       if( !header.verify_magic() )
         {
-        if( !first_member ) break;		// trailing garbage
-        pp( "Bad magic number (file not in lzip format)" );
-        retval = 2; break;
+        if( first_member )
+          { pp( "Bad magic number (file not in lzip format)" ); retval = 2; }
+        else if( verbosity >= 4 )
+          show_trailing_garbage( header.data, size, pp, false );
+        break;
         }
       if( !header.verify_version() )
         {
@@ -577,7 +619,7 @@ int decompress( const int infd, const Pretty_print & pp, const bool testing )
           header.dictionary_size() > max_dictionary_size )
         { pp( "Invalid dictionary size in member header" ); retval = 2; break; }
 
-      if( verbosity >= 1 )
+      if( verbosity >= 2 || ( verbosity == 1 && first_member ) )
         {
         pp();
         if( verbosity >= 2 )
@@ -585,8 +627,8 @@ int decompress( const int infd, const Pretty_print & pp, const bool testing )
                         header.version(),
                         format_num( header.dictionary_size() ) );
         }
-      LZ_decoder decoder( header, rdec, outfd );
 
+      LZ_decoder decoder( header, rdec, outfd );
       const int result = decoder.decode_member( pp );
       partial_file_pos += rdec.member_position();
       if( result != 0 )
@@ -603,7 +645,7 @@ int decompress( const int infd, const Pretty_print & pp, const bool testing )
           }
         retval = 2; break;
         }
-      if( verbosity >= 1 )
+      if( verbosity >= 2 )
         { if( testing ) std::fprintf( stderr, "ok\n" );
           else std::fprintf( stderr, "done\n" ); }
       }
@@ -614,18 +656,21 @@ int decompress( const int infd, const Pretty_print & pp, const bool testing )
     retval = 1;
     }
   catch( Error e ) { pp(); show_error( e.msg, errno ); retval = 1; }
+  if( verbosity == 1 && retval == 0 )
+    { if( testing ) std::fprintf( stderr, "ok\n" );
+      else std::fprintf( stderr, "done\n" ); }
   return retval;
   }
 
 
-extern "C" void signal_handler( int ) throw()
+extern "C" void signal_handler( int )
   {
   show_error( "Control-C or similar caught, quitting." );
   cleanup_and_fail( 1 );
   }
 
 
-void set_signals() throw()
+void set_signals()
   {
   std::signal( SIGHUP, signal_handler );
   std::signal( SIGINT, signal_handler );
@@ -635,24 +680,7 @@ void set_signals() throw()
 } // end namespace
 
 
-void Pretty_print::operator()( const char * const msg ) const throw()
-  {
-  if( verbosity_ >= 0 )
-    {
-    if( first_post )
-      {
-      first_post = false;
-      std::fprintf( stderr, "  %s: ", name_.c_str() );
-      for( unsigned int i = 0; i < longest_name - name_.size(); ++i )
-        std::fprintf( stderr, " " );
-      if( !msg ) std::fflush( stderr );
-      }
-    if( msg ) std::fprintf( stderr, "%s.\n", msg );
-    }
-  }
-
-
-void show_error( const char * const msg, const int errcode, const bool help ) throw()
+void show_error( const char * const msg, const int errcode, const bool help )
   {
   if( verbosity >= 0 )
     {
@@ -664,7 +692,7 @@ void show_error( const char * const msg, const int errcode, const bool help ) th
       std::fprintf( stderr, "\n" );
       }
     if( help && invocation_name && invocation_name[0] )
-      std::fprintf( stderr, "Try `%s --help' for more information.\n",
+      std::fprintf( stderr, "Try '%s --help' for more information.\n",
                     invocation_name );
     }
   }
@@ -672,48 +700,12 @@ void show_error( const char * const msg, const int errcode, const bool help ) th
 
 void internal_error( const char * const msg )
   {
-  std::fprintf( stderr, "%s: internal error: %s.\n", program_name, msg );
+  if( verbosity >= 0 )
+    std::fprintf( stderr, "%s: internal error: %s.\n", program_name, msg );
   std::exit( 3 );
   }
 
 
-// Returns the number of bytes really read.
-// If (returned value < size) and (errno == 0), means EOF was reached.
-//
-int readblock( const int fd, uint8_t * const buf, const int size ) throw()
-  {
-  int rest = size;
-  errno = 0;
-  while( rest > 0 )
-    {
-    errno = 0;
-    const int n = read( fd, buf + size - rest, rest );
-    if( n > 0 ) rest -= n;
-    else if( n == 0 ) break;
-    else if( errno != EINTR && errno != EAGAIN ) break;
-    }
-  return ( rest > 0 ) ? size - rest : size;
-  }
-
-
-// Returns the number of bytes really written.
-// If (returned value < size), it is always an error.
-//
-int writeblock( const int fd, const uint8_t * const buf, const int size ) throw()
-  {
-  int rest = size;
-  errno = 0;
-  while( rest > 0 )
-    {
-    errno = 0;
-    const int n = write( fd, buf + size - rest, rest );
-    if( n > 0 ) rest -= n;
-    else if( errno && errno != EINTR && errno != EAGAIN ) break;
-    }
-  return ( rest > 0 ) ? size - rest : size;
-  }
-
-
 int main( const int argc, const char * const argv[] )
   {
   // Mapping from gzip/bzip2 style 1..9 compression modes
@@ -737,6 +729,7 @@ int main( const int argc, const char * const argv[] )
   Mode program_mode = m_compress;
   bool force = false;
   bool keep_input_files = false;
+  bool recompress = false;
   bool to_stdout = false;
   bool zero = false;
   std::string input_filename;
@@ -759,8 +752,8 @@ int main( const int argc, const char * const argv[] )
     { 'b', "member-size",     Arg_parser::yes },
     { 'c', "stdout",          Arg_parser::no  },
     { 'd', "decompress",      Arg_parser::no  },
-    { 'e', "extreme",         Arg_parser::no  },
     { 'f', "force",           Arg_parser::no  },
+    { 'F', "recompress",      Arg_parser::no  },
     { 'h', "help",            Arg_parser::no  },
     { 'k', "keep",            Arg_parser::no  },
     { 'm', "match-length",    Arg_parser::yes },
@@ -773,7 +766,7 @@ int main( const int argc, const char * const argv[] )
     { 'V', "version",         Arg_parser::no  },
     {  0 ,  0,                Arg_parser::no  } };
 
-  Arg_parser parser( argc, argv, options );
+  const Arg_parser parser( argc, argv, options );
   if( parser.error().size() )				// bad option
     { show_error( parser.error().c_str(), 0, true ); return 1; }
 
@@ -788,35 +781,43 @@ int main( const int argc, const char * const argv[] )
       case '0': zero = true; break;
       case '1': case '2': case '3': case '4':
       case '5': case '6': case '7': case '8': case '9':
-                encoder_options = option_mapping[code-'0'];
-                zero = false; break;
-      case 'b': member_size = getnum( arg, 0, 100000, LLONG_MAX / 2 ); break;
+                zero = false;
+                encoder_options = option_mapping[code-'0']; break;
+      case 'b': member_size = getnum( arg, 100000, LLONG_MAX / 2 ); break;
       case 'c': to_stdout = true; break;
       case 'd': program_mode = m_decompress; break;
-      case 'e': break;				// ignored by now
       case 'f': force = true; break;
+      case 'F': recompress = true; break;
       case 'h': show_help(); return 0;
       case 'k': keep_input_files = true; break;
       case 'm': encoder_options.match_len_limit =
-                  getnum( arg, 0, min_match_len_limit, max_match_len );
+                  getnum( arg, min_match_len_limit, max_match_len );
                 zero = false; break;
       case 'o': default_output_filename = arg; break;
       case 'q': verbosity = -1; break;
       case 's': encoder_options.dictionary_size = get_dict_size( arg );
                 zero = false; break;
-      case 'S': volume_size = getnum( arg, 0, 100000, LLONG_MAX / 2 ); break;
+      case 'S': volume_size = getnum( arg, 100000, LLONG_MAX / 2 ); break;
       case 't': program_mode = m_test; break;
       case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version(); return 0;
       default : internal_error( "uncaught option" );
       }
-    }
+    } // end process options
 
 #if defined(__MSVCRT__) || defined(__OS2__)
-  _setmode( STDIN_FILENO, O_BINARY );
-  _setmode( STDOUT_FILENO, O_BINARY );
+  _fsetmode( stdin, "b" );
+  _fsetmode( stdout, "b" );
 #endif
 
+  if( program_mode == m_test )
+    outfd = -1;
+  else if( program_mode == m_compress )
+    {
+    dis_slots.init();
+    prob_prices.init();
+    }
+
   bool filenames_given = false;
   for( ; argind < parser.arguments(); ++argind )
     {
@@ -830,13 +831,6 @@ int main( const int argc, const char * const argv[] )
     set_signals();
 
   Pretty_print pp( filenames, verbosity );
-  if( program_mode == m_test )
-    outfd = -1;
-  else if( program_mode == m_compress )
-    {
-    dis_slots.init();
-    prob_prices.init();
-    }
 
   int retval = 0;
   for( unsigned int i = 0; i < filenames.size(); ++i )
@@ -857,7 +851,7 @@ int main( const int argc, const char * const argv[] )
           if( program_mode == m_compress )
             set_c_outname( default_output_filename, volume_size != LLONG_MAX );
           else output_filename = default_output_filename;
-          outfd_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
+          outfd_mode = all_rw;
           if( !open_outstream( force ) )
             {
             if( outfd == -1 && retval < 1 ) retval = 1;
@@ -872,7 +866,7 @@ int main( const int argc, const char * const argv[] )
       input_filename = filenames[i];
       const int eindex = extension_index( input_filename );
       infd = open_instream( input_filename, &in_stats, program_mode,
-                            eindex, force, to_stdout );
+                            eindex, recompress, to_stdout );
       if( infd < 0 ) { if( retval < 1 ) retval = 1; continue; }
       if( program_mode != m_test )
         {
@@ -882,7 +876,7 @@ int main( const int argc, const char * const argv[] )
           if( program_mode == m_compress )
             set_c_outname( input_filename, volume_size != LLONG_MAX );
           else set_d_outname( input_filename, eindex );
-          outfd_mode = S_IRUSR | S_IWUSR;
+          outfd_mode = usr_rw;
           if( !open_outstream( force ) )
             {
             if( outfd == -1 && retval < 1 ) retval = 1;
